@@ -3,7 +3,6 @@ set -e -x
 
 # Install a system package required by our library
 yum install -y numpy
-# yum install -y libgfortran
 
 ls /opt/python
 
@@ -24,10 +23,13 @@ for PYBIN in /opt/python/cp36*/bin; do
 done
 
 # Bundle external shared libraries into the wheels
-for whl in wheelhouse/*.whl; do
+ls wheelhouse/*.whl
+for whl in wheelhouse/spglib*.whl; do
     auditwheel repair "$whl" -w /io/wheelhouse/
 done
 
+cp wheelhouse/numpy*whl /io/wheelhouse/
+
 # Install packages and test
 for PYBIN in /opt/python/cp27*/bin/; do
     "${PYBIN}/pip" install spglib --no-index -f /io/wheelhouse
