@@ -49,85 +49,160 @@
 #include "debug.h"
 
 #define REDUCE_RATE 0.95
-#define NUM_ATTEMPT 20
+#define NUM_ATTEMPT 100
 #define INT_PREC 0.1
 
-static double change_of_basis_monocli[18][3][3] = {{{ 1, 0, 0 },
-                                                    { 0, 1, 0 },
-                                                    { 0, 0, 1 }},
-                                                   {{ 0, 0, 1 },
-                                                    { 0,-1, 0 },
-                                                    { 1, 0, 0 }},
-                                                   {{ 0, 0, 1 },
-                                                    { 1, 0, 0 },
-                                                    { 0, 1, 0 }},
-                                                   {{ 1, 0, 0 },
-                                                    { 0, 0, 1 },
-                                                    { 0,-1, 0 }},
-                                                   {{ 0, 1, 0 },
-                                                    { 0, 0, 1 },
-                                                    { 1, 0, 0 }},
-                                                   {{ 0,-1, 0 },
-                                                    { 1, 0, 0 },
-                                                    { 0, 0, 1 }},
-                                                   {{-1, 0, 1 },
-                                                    { 0, 1, 0 },
-                                                    {-1, 0, 0 }},
-                                                   {{ 1, 0,-1 },
-                                                    { 0,-1, 0 },
-                                                    { 0, 0,-1 }},
-                                                   {{ 0, 1,-1 },
-                                                    { 1, 0, 0 },
-                                                    { 0, 0,-1 }},
-                                                   {{-1,-1, 0 },
-                                                    { 0, 0, 1 },
-                                                    {-1, 0, 0 }},
-                                                   {{ 1,-1, 0 },
-                                                    { 0, 0, 1 },
-                                                    { 0,-1, 0 }},
-                                                   {{ 0, 1, 1 },
-                                                    { 1, 0, 0 },
-                                                    { 0, 1, 0 }},
-                                                   {{ 0, 0,-1 },
-                                                    { 0, 1, 0 },
-                                                    { 1, 0,-1 }},
-                                                   {{-1, 0, 0 },
-                                                    { 0,-1, 0 },
-                                                    {-1, 0, 1 }},
-                                                   {{ 0,-1, 0 },
-                                                    { 1, 0, 0 },
-                                                    { 0,-1, 1 }},
-                                                   {{ 0, 1, 0 },
-                                                    { 0, 0, 1 },
-                                                    { 1, 1, 0 }},
-                                                   {{-1, 0, 0 },
-                                                    { 0, 0, 1 },
-                                                    {-1, 1, 0 }},
-                                                   {{ 0, 0,-1 },
-                                                    { 1, 0, 0 },
-                                                    { 0,-1,-1 }}};
+static double change_of_basis_monocli[36][3][3] = {
+  {{ 1, 0, 0 }, /* b  first turn; two axes are flipped in second turn */
+   { 0, 1, 0 },
+   { 0, 0, 1 }},
+  {{ 0, 0, 1 }, /* b */
+   { 0,-1, 0 },
+   { 1, 0, 0 }},
+  {{ 0, 0, 1 }, /* a */
+   { 1, 0, 0 },
+   { 0, 1, 0 }},
+  {{ 1, 0, 0 }, /* c */
+   { 0, 0, 1 },
+   { 0,-1, 0 }},
+  {{ 0, 1, 0 }, /* c */
+   { 0, 0, 1 },
+   { 1, 0, 0 }},
+  {{ 0,-1, 0 }, /* a */
+   { 1, 0, 0 },
+   { 0, 0, 1 }},
+  {{-1, 0, 1 }, /* b */
+   { 0, 1, 0 },
+   {-1, 0, 0 }},
+  {{ 1, 0,-1 }, /* b */
+   { 0,-1, 0 },
+   { 0, 0,-1 }},
+  {{ 0, 1,-1 }, /* a */
+   { 1, 0, 0 },
+   { 0, 0,-1 }},
+  {{-1,-1, 0 }, /* c */
+   { 0, 0, 1 },
+   {-1, 0, 0 }},
+  {{ 1,-1, 0 }, /* c */
+   { 0, 0, 1 },
+   { 0,-1, 0 }},
+  {{ 0, 1, 1 }, /* a */
+   { 1, 0, 0 },
+   { 0, 1, 0 }},
+  {{ 0, 0,-1 }, /* b */
+   { 0, 1, 0 },
+   { 1, 0,-1 }},
+  {{-1, 0, 0 }, /* b */
+   { 0,-1, 0 },
+   {-1, 0, 1 }},
+  {{ 0,-1, 0 }, /* a */
+   { 1, 0, 0 },
+   { 0,-1, 1 }},
+  {{ 0, 1, 0 }, /* c */
+   { 0, 0, 1 },
+   { 1, 1, 0 }},
+  {{-1, 0, 0 }, /* c */
+   { 0, 0, 1 },
+   {-1, 1, 0 }},
+  {{ 0, 0,-1 }, /* a */
+   { 1, 0, 0 },
+   { 0,-1,-1 }},
+  {{ 1, 0, 0 }, /* b  two axes are flipped to look for non-acute axes */
+   { 0,-1, 0 },
+   { 0, 0,-1 }},
+  {{ 0, 0,-1 }, /* b */
+   { 0, 1, 0 },
+   { 1, 0, 0 }},
+  {{ 0, 0, 1 }, /* a */
+   {-1, 0, 0 },
+   { 0,-1, 0 }},
+  {{-1, 0, 0 }, /* c */
+   { 0, 0,-1 },
+   { 0,-1, 0 }},
+  {{ 0, 1, 0 }, /* c */
+   { 0, 0,-1 },
+   {-1, 0, 0 }},
+  {{ 0, 1, 0 }, /* a */
+   {-1, 0, 0 },
+   { 0, 0, 1 }},
+  {{-1, 0,-1 }, /* b */
+   { 0,-1, 0 },
+   {-1, 0, 0 }},
+  {{ 1, 0, 1 }, /* b */
+   { 0, 1, 0 },
+   { 0, 0, 1 }},
+  {{ 0,-1,-1 }, /* a */
+   {-1, 0, 0 },
+   { 0, 0,-1 }},
+  {{ 1,-1, 0 }, /* c */
+   { 0, 0,-1 },
+   { 1, 0, 0 }},
+  {{-1,-1, 0 }, /* c */
+   { 0, 0,-1 },
+   { 0,-1, 0 }},
+  {{ 0,-1, 1 }, /* a */
+   {-1, 0, 0 },
+   { 0,-1, 0 }},
+  {{ 0, 0, 1 }, /* b */
+   { 0,-1, 0 },
+   { 1, 0, 1 }},
+  {{-1, 0, 0 }, /* b */
+   { 0, 1, 0 },
+   {-1, 0,-1 }},
+  {{ 0, 1, 0 }, /* a */
+   {-1, 0, 0 },
+   { 0, 1, 1 }},
+  {{ 0, 1, 0 }, /* c */
+   { 0, 0,-1 },
+   {-1, 1, 0 }},
+  {{ 1, 0, 0 }, /* c */
+   { 0, 0,-1 },
+   { 1, 1, 0 }},
+  {{ 0, 0,-1 }, /* a */
+   {-1, 0, 0 },
+   { 0, 1,-1 }}};
 
-static Centering change_of_centering_monocli[18] = {C_FACE,
-                                                    A_FACE,
-                                                    B_FACE,
-                                                    B_FACE,
-                                                    A_FACE,
-                                                    C_FACE,
-                                                    BASE,
-                                                    BASE,
-                                                    BASE,
-                                                    BASE,
-                                                    BASE,
-                                                    BASE,
-                                                    A_FACE,
-                                                    C_FACE,
-                                                    C_FACE,
-                                                    A_FACE,
-                                                    B_FACE,
-                                                    B_FACE};
+static Centering change_of_centering_monocli[36] = {
+  C_FACE, /* first turn */
+  A_FACE,
+  B_FACE,
+  B_FACE,
+  A_FACE,
+  C_FACE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  A_FACE,
+  C_FACE,
+  C_FACE,
+  A_FACE,
+  B_FACE,
+  B_FACE,
+  C_FACE, /* second turn */
+  A_FACE,
+  B_FACE,
+  B_FACE,
+  A_FACE,
+  C_FACE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  BASE,
+  A_FACE,
+  C_FACE,
+  C_FACE,
+  A_FACE,
+  B_FACE,
+  B_FACE};
 
-static int change_of_unique_axis_monocli[18] =
-  {1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0};
+static int change_of_unique_axis_monocli[36] = {
+  1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0,
+  1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0, 1, 1, 0, 2, 2, 0};
 
 static double change_of_basis_ortho[6][3][3] = {{{ 1, 0, 0 },
                                                  { 0, 1, 0 },
@@ -227,15 +302,15 @@ static double F_mat[3][3] = {{    0, 1./2, 1./2 },
                              { 1./2,    0, 1./2 },
                              { 1./2, 1./2,    0 }};
 
-static Spacegroup search_spacegroup_with_symmetry(const Cell * primitive,
-                                                  const int candidates[],
-                                                  const int num_candidates,
-                                                  const Symmetry *symmetry,
-                                                  const double symprec,
-                                                  const double angle_tolerance);
-static Spacegroup get_spacegroup(const int hall_number,
-                                 const double origin_shift[3],
-                                 SPGCONST double conv_lattice[3][3]);
+static Spacegroup * search_spacegroup_with_symmetry(const Cell * primitive,
+                                                    const int candidates[],
+                                                    const int num_candidates,
+                                                    const Symmetry *symmetry,
+                                                    const double symprec,
+                                                    const double angle_tolerance);
+static Spacegroup * get_spacegroup(const int hall_number,
+                                   const double origin_shift[3],
+                                   SPGCONST double conv_lattice[3][3]);
 static int iterative_search_hall_number(double origin_shift[3],
                                         double conv_lattice[3][3],
                                         const int candidates[],
@@ -297,23 +372,23 @@ static int get_centering_shifts(double shift[3][3],
 
 
 /* Return spacegroup.number = 0 if failed */
-Spacegroup spa_search_spacegroup(const Cell * primitive,
-                                 const int hall_number,
-                                 const double symprec,
-                                 const double angle_tolerance)
+Spacegroup * spa_search_spacegroup(const Cell * primitive,
+                                   const int hall_number,
+                                   const double symprec,
+                                   const double angle_tolerance)
 {
-  Spacegroup spacegroup;
+  Spacegroup *spacegroup;
   Symmetry *symmetry;
   int candidate[1];
 
   debug_print("search_spacegroup (tolerance = %f):\n", symprec);
 
   symmetry = NULL;
-  spacegroup.number = 0;
+  spacegroup = NULL;
 
   if ((symmetry = sym_get_operation(primitive, symprec, angle_tolerance)) ==
       NULL) {
-    goto ret;
+    return NULL;
   }
 
   if (hall_number > 0) {
@@ -339,18 +414,19 @@ Spacegroup spa_search_spacegroup(const Cell * primitive,
   sym_free_symmetry(symmetry);
   symmetry = NULL;
 
- ret:
   return spacegroup;
 }
 
-
-Spacegroup spa_search_spacegroup_with_symmetry(const Symmetry *symmetry,
-                                               const double symprec)
+/* Retrun 0 if failed */
+int spa_search_spacegroup_with_symmetry(const Symmetry *symmetry,
+                                        const double symprec)
 {
-  int i;
-  Spacegroup spacegroup;
+  int i, hall_number;
+  Spacegroup *spacegroup;
   Cell *primitive;
 
+  spacegroup = NULL;
+
   primitive = cel_alloc_cell(1);
   mat_copy_matrix_d3(primitive->lattice, identity);
   for (i = 0; i < 3; i++) {
@@ -362,7 +438,14 @@ Spacegroup spa_search_spacegroup_with_symmetry(const Symmetry *symmetry,
                                                symmetry,
                                                symprec,
                                                -1.0);
-  return spacegroup;
+  if (spacegroup != NULL) {
+    hall_number = spacegroup->hall_number;
+    free(spacegroup);
+    spacegroup = NULL;
+    return hall_number;
+  } else {
+    return 0;
+  }
 }
 
 /* Return NULL if failed */
@@ -508,30 +591,29 @@ Cell * spa_transform_from_primitive(const Cell * primitive,
   return trimmed_cell;
 }
 
-/* Return spacegroup.number = 0 if failed */
-static Spacegroup search_spacegroup_with_symmetry(const Cell * primitive,
-                                                  const int candidates[],
-                                                  const int num_candidates,
-                                                  const Symmetry *symmetry,
-                                                  const double symprec,
-                                                  const double angle_tolerance)
+/* Return NULL if failed */
+static Spacegroup * search_spacegroup_with_symmetry(const Cell * primitive,
+                                                    const int candidates[],
+                                                    const int num_candidates,
+                                                    const Symmetry *symmetry,
+                                                    const double symprec,
+                                                    const double angle_tolerance)
 {
   int hall_number;
   double conv_lattice[3][3];
   double origin_shift[3];
-  Spacegroup spacegroup;
+  Spacegroup *spacegroup;
   PointSymmetry pointsym;
 
   debug_print("search_spacegroup (tolerance = %f):\n", symprec);
 
-  hall_number = 0;
-  spacegroup.number = 0;
+  spacegroup = NULL;
 
   pointsym = ptg_get_pointsymmetry(symmetry->rot, symmetry->size);
   if (pointsym.size < symmetry->size) {
     warning_print("spglib: Point symmetry of primitive cell is broken. ");
     warning_print("(line %d, %s).\n", __LINE__, __FILE__);
-    goto ret;
+    return NULL;
   }
 
   hall_number = iterative_search_hall_number(origin_shift,
@@ -542,41 +624,44 @@ static Spacegroup search_spacegroup_with_symmetry(const Cell * primitive,
                                              symmetry,
                                              symprec,
                                              angle_tolerance);
+  if (hall_number == 0) {
+    return NULL;
+  }
+
   spacegroup = get_spacegroup(hall_number, origin_shift, conv_lattice);
 
- ret:
   return spacegroup;
 }
 
 /* Return spacegroup.number = 0 if failed */
-static Spacegroup get_spacegroup(const int hall_number,
-                                 const double origin_shift[3],
-                                 SPGCONST double conv_lattice[3][3])
+static Spacegroup * get_spacegroup(const int hall_number,
+                                   const double origin_shift[3],
+                                   SPGCONST double conv_lattice[3][3])
 {
-  Spacegroup spacegroup;
+  Spacegroup *spacegroup;
   SpacegroupType spacegroup_type;
 
-  spacegroup.number = 0;
-  spacegroup_type = spgdb_get_spacegroup_type(hall_number);
+  spacegroup = NULL;
 
-  if (spacegroup_type.number > 0) {
-    mat_copy_matrix_d3(spacegroup.bravais_lattice, conv_lattice);
-    mat_copy_vector_d3(spacegroup.origin_shift, origin_shift);
-    spacegroup.number = spacegroup_type.number;
-    spacegroup.hall_number = hall_number;
-    spacegroup.pointgroup_number = spacegroup_type.pointgroup_number;
-    strcpy(spacegroup.schoenflies,
-           spacegroup_type.schoenflies);
-    strcpy(spacegroup.hall_symbol,
-           spacegroup_type.hall_symbol);
-    strcpy(spacegroup.international,
-           spacegroup_type.international);
-    strcpy(spacegroup.international_long,
-           spacegroup_type.international_full);
-    strcpy(spacegroup.international_short,
+  if ((spacegroup = (Spacegroup*) malloc(sizeof(Spacegroup))) == NULL) {
+    warning_print("spglib: Memory could not be allocated.");
+    return NULL;
+  }
+
+  if (0 < hall_number && hall_number < 531) {
+    spacegroup_type = spgdb_get_spacegroup_type(hall_number);
+    mat_copy_matrix_d3(spacegroup->bravais_lattice, conv_lattice);
+    mat_copy_vector_d3(spacegroup->origin_shift, origin_shift);
+    spacegroup->number = spacegroup_type.number;
+    spacegroup->hall_number = hall_number;
+    spacegroup->pointgroup_number = spacegroup_type.pointgroup_number;
+    strcpy(spacegroup->schoenflies, spacegroup_type.schoenflies);
+    strcpy(spacegroup->hall_symbol, spacegroup_type.hall_symbol);
+    strcpy(spacegroup->international, spacegroup_type.international);
+    strcpy(spacegroup->international_long, spacegroup_type.international_full);
+    strcpy(spacegroup->international_short,
            spacegroup_type.international_short);
-    strcpy(spacegroup.choice,
-           spacegroup_type.choice);
+    strcpy(spacegroup->choice, spacegroup_type.choice);
   }
 
   return spacegroup;
@@ -616,7 +701,7 @@ static int iterative_search_hall_number(double origin_shift[3],
   tolerance = symprec;
   for (attempt = 0; attempt < NUM_ATTEMPT; attempt++) {
 
-    warning_print("spglib: Attempt %d tolerance = %f failed",
+    warning_print("spglib: Attempt %d tolerance = %e failed",
                   attempt, tolerance);
     warning_print("(line %d, %s).\n", __LINE__, __FILE__);
 
@@ -1031,14 +1116,15 @@ static int match_hall_symbol_db_monocli(double origin_shift[3],
                                         const double symprec)
 {
   int i, j, k, l, is_found;
-  double vec[3], norms[3];
+  double vec[2][3], norms[3];
   Centering changed_centering;
   Symmetry * changed_symmetry;
   double changed_lattice[3][3];
 
   changed_symmetry = NULL;
 
-  for (i = 0; i < 18; i++) {
+  for (i = 0; i < 36; i++) {
+    /* centring type should be P or C */
     if (centering == C_FACE) {
       changed_centering = change_of_centering_monocli[i];
     } else { /* suppose PRIMITIVE */
@@ -1049,18 +1135,28 @@ static int match_hall_symbol_db_monocli(double origin_shift[3],
                            lattice,
                            change_of_basis_monocli[i]);
 
+    /* Make non-acute and length preference */
+    l = 0;
+    for (j = 0; j < 3; j++) {
+      if (j == change_of_unique_axis_monocli[i]) {continue;}
+      for (k = 0; k < 3; k++) {vec[l][k] = changed_lattice[k][j];}
+      norms[l] = mat_norm_squared_d3(vec[l]);
+      l++;
+    }
+
+    /* discard if principal angle is acute. */
+    if (vec[0][0] * vec[1][0] +
+        vec[0][1] * vec[1][1] +
+        vec[0][2] * vec[1][2] > 0) {
+      continue;
+    }
+
     /* Choose |a| < |b| < |c| if there are freedom. */
     if (num_hall_types == 3) {
-      l = 0;
-      for (j = 0; j < 3; j++) {
-        if (j == change_of_unique_axis_monocli[i]) {continue;}
-        for (k = 0; k < 3; k++) {vec[k] = changed_lattice[k][j];}
-        norms[l] = mat_norm_squared_d3(vec);
-        l++;
-      }
       if (norms[0] > norms[1]) {continue;}
     }
 
+
     if ((changed_symmetry =
          get_conventional_symmetry(change_of_basis_monocli[i],
                                    PRIMITIVE,
@@ -1359,9 +1455,15 @@ static Centering get_base_center(SPGCONST int transform_mat[3][3])
   }
 
   /* body center */
-  if (abs(transform_mat[0][0]) +
-      abs(transform_mat[0][1]) +
-      abs(transform_mat[0][2]) == 2) {
+  if ((abs(transform_mat[0][0]) +
+       abs(transform_mat[0][1]) +
+       abs(transform_mat[0][2]) == 2) &&
+      (abs(transform_mat[1][0]) +
+       abs(transform_mat[1][1]) +
+       abs(transform_mat[1][2]) == 2) &&
+      (abs(transform_mat[2][0]) +
+       abs(transform_mat[2][1]) +
+       abs(transform_mat[2][2]) == 2)) {
     centering = BODY;
     goto end;
   }
