@@ -163,7 +163,7 @@ static int get_schoenflies(char symbol[7],
 /* kpoints */
 /*---------*/
 static int get_ir_reciprocal_mesh(int grid_address[][3],
-                                  int map[],
+                                  int ir_mapping_table[],
                                   const int mesh[3],
                                   const int is_shift[3],
                                   const int is_time_reversal,
@@ -173,9 +173,20 @@ static int get_ir_reciprocal_mesh(int grid_address[][3],
                                   const int num_atom,
                                   const double symprec,
                                   const double angle_tolerance);
+static long get_long_ir_reciprocal_mesh(int grid_address[][3],
+                                        long ir_mapping_table[],
+                                        const int mesh[3],
+                                        const int is_shift[3],
+                                        const int is_time_reversal,
+                                        SPGCONST double lattice[3][3],
+                                        SPGCONST double position[][3],
+                                        const int types[],
+                                        const int num_atom,
+                                        const double symprec,
+                                        const double angle_tolerance);
 
 static int get_stabilized_reciprocal_mesh(int grid_address[][3],
-                                          int map[],
+                                          int ir_mapping_table[],
                                           const int mesh[3],
                                           const int is_shift[3],
                                           const int is_time_reversal,
@@ -183,6 +194,15 @@ static int get_stabilized_reciprocal_mesh(int grid_address[][3],
                                           SPGCONST int rotations[][3][3],
                                           const int num_q,
                                           SPGCONST double qpoints[][3]);
+static long get_long_stabilized_reciprocal_mesh(int grid_address[][3],
+                                                long ir_mapping_table[],
+                                                const int mesh[3],
+                                                const int is_shift[3],
+                                                const int is_time_reversal,
+                                                const int num_rot,
+                                                SPGCONST int rotations[][3][3],
+                                                const int num_q,
+                                                SPGCONST double qpoints[][3]);
 
 /*========*/
 /* global */
@@ -841,8 +861,24 @@ int spg_get_grid_point_from_address(const int grid_address[3],
   return kgd_get_grid_point_double_mesh(address_double, mesh);
 }
 
+long spg_get_long_grid_point_from_address(const int grid_address[3],
+                                          const int mesh[3])
+{
+  int address_double[3];
+  int is_shift[3];
+
+  is_shift[0] = 0;
+  is_shift[1] = 0;
+  is_shift[2] = 0;
+  kgd_get_grid_address_double_mesh(address_double,
+                                   grid_address,
+                                   mesh,
+                                   is_shift);
+  return kgd_get_long_grid_point_double_mesh(address_double, mesh);
+}
+
 int spg_get_ir_reciprocal_mesh(int grid_address[][3],
-                               int map[],
+                               int ir_mapping_table[],
                                const int mesh[3],
                                const int is_shift[3],
                                const int is_time_reversal,
@@ -853,7 +889,7 @@ int spg_get_ir_reciprocal_mesh(int grid_address[][3],
                                const double symprec)
 {
   return get_ir_reciprocal_mesh(grid_address,
-                                map,
+                                ir_mapping_table,
                                 mesh,
                                 is_shift,
                                 is_time_reversal,
@@ -865,8 +901,32 @@ int spg_get_ir_reciprocal_mesh(int grid_address[][3],
                                 -1.0);
 }
 
+long spg_get_long_ir_reciprocal_mesh(int grid_address[][3],
+                                     long ir_mapping_table[],
+                                     const int mesh[3],
+                                     const int is_shift[3],
+                                     const int is_time_reversal,
+                                     SPGCONST double lattice[3][3],
+                                     SPGCONST double position[][3],
+                                     const int types[],
+                                     const int num_atom,
+                                     const double symprec)
+{
+  return get_long_ir_reciprocal_mesh(grid_address,
+                                     ir_mapping_table,
+                                     mesh,
+                                     is_shift,
+                                     is_time_reversal,
+                                     lattice,
+                                     position,
+                                     types,
+                                     num_atom,
+                                     symprec,
+                                     -1.0);
+}
+
 int spg_get_stabilized_reciprocal_mesh(int grid_address[][3],
-                                       int map[],
+                                       int ir_mapping_table[],
                                        const int mesh[3],
                                        const int is_shift[3],
                                        const int is_time_reversal,
@@ -876,7 +936,7 @@ int spg_get_stabilized_reciprocal_mesh(int grid_address[][3],
                                        SPGCONST double qpoints[][3])
 {
   return get_stabilized_reciprocal_mesh(grid_address,
-                                        map,
+                                        ir_mapping_table,
                                         mesh,
                                         is_shift,
                                         is_time_reversal,
@@ -886,6 +946,27 @@ int spg_get_stabilized_reciprocal_mesh(int grid_address[][3],
                                         qpoints);
 }
 
+long spg_get_long_stabilized_reciprocal_mesh(int grid_address[][3],
+                                             long ir_mapping_table[],
+                                             const int mesh[3],
+                                             const int is_shift[3],
+                                             const int is_time_reversal,
+                                             const int num_rot,
+                                             SPGCONST int rotations[][3][3],
+                                             const int num_q,
+                                             SPGCONST double qpoints[][3])
+{
+  return get_long_stabilized_reciprocal_mesh(grid_address,
+                                             ir_mapping_table,
+                                             mesh,
+                                             is_shift,
+                                             is_time_reversal,
+                                             num_rot,
+                                             rotations,
+                                             num_q,
+                                             qpoints);
+}
+
 int spg_get_grid_points_by_rotations(int rot_grid_points[],
                                      const int address_orig[3],
                                      const int num_rot,
@@ -916,6 +997,36 @@ int spg_get_grid_points_by_rotations(int rot_grid_points[],
   return 1;
 }
 
+int spg_get_long_grid_points_by_rotations(long rot_grid_points[],
+                                          const int address_orig[3],
+                                          const int num_rot,
+                                          SPGCONST int rot_reciprocal[][3][3],
+                                          const int mesh[3],
+                                          const int is_shift[3])
+{
+  int i;
+  MatINT *rot;
+
+  rot = NULL;
+
+  if ((rot = mat_alloc_MatINT(num_rot)) == NULL) {
+    return 0;
+  }
+
+  for (i = 0; i < num_rot; i++) {
+    mat_copy_matrix_i3(rot->mat[i], rot_reciprocal[i]);
+  }
+  kpt_get_long_grid_points_by_rotations(rot_grid_points,
+                                        address_orig,
+                                        rot,
+                                        mesh,
+                                        is_shift);
+  mat_free_MatINT(rot);
+  rot = NULL;
+
+  return 1;
+}
+
 int spg_get_BZ_grid_points_by_rotations(int rot_grid_points[],
                                         const int address_orig[3],
                                         const int num_rot,
@@ -948,6 +1059,38 @@ int spg_get_BZ_grid_points_by_rotations(int rot_grid_points[],
   return 1;
 }
 
+int spg_get_long_BZ_grid_points_by_rotations(long rot_grid_points[],
+                                             const int address_orig[3],
+                                             const int num_rot,
+                                             SPGCONST int rot_reciprocal[][3][3],
+                                             const int mesh[3],
+                                             const int is_shift[3],
+                                             const long bz_map[])
+{
+  int i;
+  MatINT *rot;
+
+  rot = NULL;
+
+  if ((rot = mat_alloc_MatINT(num_rot)) == NULL) {
+    return 0;
+  }
+
+  for (i = 0; i < num_rot; i++) {
+    mat_copy_matrix_i3(rot->mat[i], rot_reciprocal[i]);
+  }
+  kpt_get_long_BZ_grid_points_by_rotations(rot_grid_points,
+                                           address_orig,
+                                           rot,
+                                           mesh,
+                                           is_shift,
+                                           bz_map);
+  mat_free_MatINT(rot);
+  rot = NULL;
+
+  return 1;
+}
+
 int spg_relocate_BZ_grid_address(int bz_grid_address[][3],
                                  int bz_map[],
                                  SPGCONST int grid_address[][3],
@@ -963,6 +1106,21 @@ int spg_relocate_BZ_grid_address(int bz_grid_address[][3],
                                       is_shift);
 }
 
+long spg_relocate_long_BZ_grid_address(int bz_grid_address[][3],
+                                       long bz_map[],
+                                       SPGCONST int grid_address[][3],
+                                       const int mesh[3],
+                                       SPGCONST double rec_lattice[3][3],
+                                       const int is_shift[3])
+{
+  return kpt_relocate_long_BZ_grid_address(bz_grid_address,
+                                           bz_map,
+                                           grid_address,
+                                           mesh,
+                                           rec_lattice,
+                                           is_shift);
+}
+
 /*--------*/
 /* Niggli */
 /*--------*/
@@ -1864,7 +2022,7 @@ static int get_schoenflies(char symbol[7],
 /* kpoints */
 /*---------*/
 static int get_ir_reciprocal_mesh(int grid_address[][3],
-                                  int map[],
+                                  int ir_mapping_table[],
                                   const int mesh[3],
                                   const int is_shift[3],
                                   const int is_time_reversal,
@@ -1900,7 +2058,7 @@ static int get_ir_reciprocal_mesh(int grid_address[][3],
   }
   rot_reciprocal = kpt_get_point_group_reciprocal(rotations, is_time_reversal);
   num_ir = kpt_get_irreducible_reciprocal_mesh(grid_address,
-                                               map,
+                                               ir_mapping_table,
                                                mesh,
                                                is_shift,
                                                rot_reciprocal);
@@ -1913,6 +2071,57 @@ static int get_ir_reciprocal_mesh(int grid_address[][3],
   return num_ir;
 }
 
+static long get_long_ir_reciprocal_mesh(int grid_address[][3],
+                                        long ir_mapping_table[],
+                                        const int mesh[3],
+                                        const int is_shift[3],
+                                        const int is_time_reversal,
+                                        SPGCONST double lattice[3][3],
+                                        SPGCONST double position[][3],
+                                        const int types[],
+                                        const int num_atom,
+                                        const double symprec,
+                                        const double angle_tolerance)
+{
+  SpglibDataset *dataset;
+  int i;
+  long num_ir;
+  MatINT *rotations, *rot_reciprocal;
+
+  if ((dataset = get_dataset(lattice,
+                             position,
+                             types,
+                             num_atom,
+                             0,
+                             symprec,
+                             angle_tolerance)) == NULL) {
+    return 0;
+  }
+
+  if ((rotations = mat_alloc_MatINT(dataset->n_operations)) == NULL) {
+    spg_free_dataset(dataset);
+    dataset = NULL;
+    return 0;
+  }
+
+  for (i = 0; i < dataset->n_operations; i++) {
+    mat_copy_matrix_i3(rotations->mat[i], dataset->rotations[i]);
+  }
+  rot_reciprocal = kpt_get_point_group_reciprocal(rotations, is_time_reversal);
+  num_ir = kpt_get_long_irreducible_reciprocal_mesh(grid_address,
+                                                    ir_mapping_table,
+                                                    mesh,
+                                                    is_shift,
+                                                    rot_reciprocal);
+  mat_free_MatINT(rot_reciprocal);
+  rot_reciprocal = NULL;
+  mat_free_MatINT(rotations);
+  rotations = NULL;
+  spg_free_dataset(dataset);
+  dataset = NULL;
+  return num_ir;
+}
+
 static int get_stabilized_reciprocal_mesh(int grid_address[][3],
                                           int map[],
                                           const int mesh[3],
@@ -1950,3 +2159,42 @@ static int get_stabilized_reciprocal_mesh(int grid_address[][3],
 
   return num_ir;
 }
+
+static long get_long_stabilized_reciprocal_mesh(int grid_address[][3],
+                                                long ir_mapping_table[],
+                                                const int mesh[3],
+                                                const int is_shift[3],
+                                                const int is_time_reversal,
+                                                const int num_rot,
+                                                SPGCONST int rotations[][3][3],
+                                                const int num_q,
+                                                SPGCONST double qpoints[][3])
+{
+  MatINT *rot_real;
+  int i;
+  long num_ir;
+
+  rot_real = NULL;
+
+  if ((rot_real = mat_alloc_MatINT(num_rot)) == NULL) {
+    return 0;
+  }
+
+  for (i = 0; i < num_rot; i++) {
+    mat_copy_matrix_i3(rot_real->mat[i], rotations[i]);
+  }
+
+  num_ir = kpt_get_long_stabilized_reciprocal_mesh(grid_address,
+                                                   ir_mapping_table,
+                                                   mesh,
+                                                   is_shift,
+                                                   is_time_reversal,
+                                                   rot_real,
+                                                   num_q,
+                                                   qpoints);
+
+  mat_free_MatINT(rot_real);
+  rot_real = NULL;
+
+  return num_ir;
+}
